%Generating S_T conditional on sig, p, y_T
function [ST, S1T, S2T] = gbs_st(param, xdatae)

mu = param(5);

y = xdatae(2:end);
x = xdatae(1:end-1);

t = size(xdatae,1)-1;

%Initial probability Pr[S0/YO]
p11=param(1); p21=1-p11;
p22=param(2); p12=1-p22;

Q = [p11 p12          
     p21 p22]; %Transition matrix
     
     
%Steady-state probabilities PR[S_t=1] and PR[S_t=2]
A = [(eye(2)-Q);ones(1,2)];
EN=[0;0;1];
Prob_ss = (inv(A'*A))*A'*EN; %3x1 vector steady state probabilities 
Prob1   = Prob_ss;

sig = [param(3); param(4)];
ss = [sig; sig];
flt_pr = zeros(2,t);

for j=1:t
   Pr = vec(Q).*vecr([Prob_ss Prob_ss]); % 9x1 vector
   f=(y(j,1)-x(j,:)*mu)*ones(4,1); % function
   densj=(1./sqrt(2*pi*ss)).*exp(-0.5*f.*f./ss).*Pr; %joint density
%The density of y_t given past information I(t-1) : f(y_t|I(t-1)): the
%marginal density of y_t
dens= sum(densj); %weighted average of conditional densities

%updating the probability terms with new information y_t
%Pr[S_t,S_{t-1} | I(t-1),y_t] = Pr[S_t,S_{t-1} | I(t)]
densj_s_new=densj/dens;
Prob_ss = densj_s_new(1:2,1)+densj_s_new(3:4,1); %Integrate out S_{t-1}:
                                                % Pr[S_t | Y_t]                                                
flt_pr(:,j) = Prob_ss; %store each Pr[S_t=1 | Y_t]
end                               %Pr[S_t=2 | Y_t]
                                  
% Add the steady state probabilities
flt_pr = [Prob1 flt_pr];
                                  
%Generate S_T based on Carter and Kohn (1994)
ST=zeros(t+1,1); STmat=zeros(t+1,2); 
ST(t+1,1)= bingen(flt_pr(1,t+1), flt_pr(2,t+1));
STmat(t+1,ST(t+1,1)+1)=1;

for i=t:-1:1 % backward recursion (i=t-1:-1:1)
    
if ST(i+1,1)==0;
                    p0=p11*flt_pr(1,i);
                    p1=(1-p22)*flt_pr(2,i);
elseif ST(i+1,1)==1;
                    p0=(1-p11)*flt_pr(1,i);
                    p1=p22*flt_pr(2,i);
end

ST(i,1)=bingen(p0,p1);
STmat(i,ST(i,1)+1)=1;

end

S1T=STmat(:,1); S2T=STmat(:,2);

% figure
% plot(flt_pr(1,:))
