function [drawsMat, rej] = gen_mu(drawsMat, jj, xdd, param, postheta, sigscale, cc, ndraws, ...
                                                 lb, ub, ST, sig, pmean, pstdd, pshape, rej, nparam, i_block, other_specifications, idx_specification, options_)
      
% Random-Walk Metropolis-Hasting Algorithm 
%
% Stephane Lhuissier, 2015

% Generate Candidate
xcand = drawsMat(:,jj-1) + cc*(sigscale*randn(nparam,1));

% check bounds
ind1 = find(xcand > ub(postheta));
ind2 = find(xcand < lb(postheta));

if any(ind1) || any(ind2)
	logpostNEW = -1e10;
else
    logpostNEW = ms_post_fc_conditional_path(xcand, options_.param, options_.postheta, options_.possig, options_.posq, xdd, ST, sig, other_specifications, idx_specification, options_);
end

logpostOLD = ms_post_fc_conditional_path(drawsMat(:,jj-1), options_.param, options_.postheta, options_.possig, options_.posq, xdd, ST, sig, other_specifications, idx_specification, options_);

r = min([1 ; exp(logpostNEW - logpostOLD)]);

if rand(1,1) < r  
	drawsMat(:,jj)=xcand;
else
	drawsMat(:,jj)=drawsMat(:,jj-1);
	rej(jj)=1;
end                      

fprintf(1,'\nSimulation step %2.0f of %2.0f [block no. %d]',[jj,ndraws,i_block]);
fprintf(1,'\nRejection r: %2.6f rej %2.2g perct %2.4f [block no. %d]',[r,rej(jj),sum(rej)/jj,i_block]);

