function parasim = gen_sig(ST, xddsim, F, impact, a_invgam, b_invgam, nstates)
% generate structural shocks
%
%-----------------------------------------------------------------------------------
%------------------------ Inverse-Gamma distribution ------------------------------%
%--- p(x) = ( b^a/Gamma(a) ) x^(-a-1) exp(-b/x) for a>0 and b>0.
%---    where a is shape and b is scale parameter.
%--- E(x) = b/(a-1) for a>1;  var(x) = b^2/( (a-1)^2*(a-2) ) for a>2;
%--- Noninformative distribution: a,b -> 0.
%--- How to draw: (1) draw z from Gamma(a,b); (2) let x=1/z.
%-----------------------------------------------------------------------------------
% for more info, see Tao Zha notes.

% by Stphane Lhuissier, 2015

nshocks = size(impact,2);
parasim = zeros(nshocks,nstates);

% structural shocks, nshocks x T
e = pinv(impact)*(xddsim(2:end,:)'-F*xddsim(1:end-1,:)');

% remove one period
ST = ST(2:end);

idx_accept = 0;

while idx_accept == 0
k = 1;         
% draws from inverse-gamma distribution
for ns=1:nstates 
    RR = find(ST==ns);
    
    for jj=1:nshocks
        
        ee = e(jj,RR);
        t = size(ee,2);

%        a = 0 + 1/2*t;
%        b = 0 + 1/2*(ee*ee');
         a = a_invgam(k) + 1/2*t;
         b = b_invgam(k) + 1/2*(ee*ee');

        parasim(jj,ns) = sqrt(1./gamrnd(a,1/b)); %Note that in Matlab, it is 1/b, NOT b.
        k = k + 1; 
        
    end
end

    % Normalization rule, see HWZ
    eval(['ftd_',num2str(nstates),'v']);
    
end