function Z_MAT = gen_zt(YYSTAR, AAA, H, RR, impact, SDX, MU, ST, nstates, cv)
% Generate DSGE state vector 
%
% y(t) = H*b(t) + e(t) where E[e(t)*e(t)']=R
% b(t) = F*b(t-1) + mu(t) where E[mu(t)*mu(t)']=Q
%
% by Stephane Lhuissier, 2015

NY=size(AAA,1);
t=size(YYSTAR,1); %-1;
TSTAR=t;

% variance-covariance matrix of the disturbance vector
Q = zeros(NY,NY,nstates);
for ii=1:nstates
    Q(:,:,ii) = impact*(SDX(:,:,ii)^2)*impact';
end

% Multi-move Gibbs sampling by Carter and Kohn (1994)
%----------------------------------------------------------------
% STEP 1.a Recovering the last iteration [beta_tt and P_tt] of kalman filter
[~, XTT_M, PTT_M] = kf_conditional_path(YYSTAR, AAA, Q, H, MU, RR, ST);

% STEP 1.b Drawing state vector, [beta_TT and P_TT] 
Z_MAT = zeros(TSTAR,NY);
X_TMP = XTT_M(:,end);
P_TMP = PTT_M(:,:,end);

G  = real(sqrtm(P_TMP));
ZT = G*normrnd(0,1,NY,1)+X_TMP;
Z_MAT(TSTAR,:) = ZT;

% STEP 2. For t = T-1, T-2, ..., 1, drawing state vector [beta_tt and P_tt]
J_ITER = TSTAR-1; 

while J_ITER>=1 
    
    F_CAST0 = ZT - AAA*XTT_M(:,J_ITER);

    SS0 = AAA*PTT_M(:,:,J_ITER)*AAA' + Q(:,:,ST(J_ITER+1));

    K_GN0 =PTT_M(:,:,J_ITER)*AAA'*pinv(SS0,cv);

    XTT_M(:,J_ITER) = XTT_M(:,J_ITER) + K_GN0*F_CAST0;
        
    PTT_M(:,:,J_ITER) = (eye(NY) - K_GN0*AAA)*PTT_M(:,:,J_ITER);       
    
    X_TMP = XTT_M(:,J_ITER);
    P_TMP = PTT_M(:,:,J_ITER);

    G  = real(sqrtm(P_TMP));
    ZT = G*normrnd(0,1,NY,1)+X_TMP;
    Z_MAT(J_ITER,:)=ZT;

    J_ITER=J_ITER-1;  
    
end

