function [lnlik, store_beta, store_P, Q] = kf_conditional_path(xdd, F, Q, H, MU, R, ST)
% Compute the conditional likelihood on the path of stochastic volatilities
% 
% by Stephane Lhuissier, 2015

t       = size(xdd,1);
nvarobs = size(xdd,2);
t0      = 4;
NY      = size(F,1);

% Initializing Kalman Filter
%--------------------------------------------------------------
beta_ll = zeros(NY,1);
P_ll    = dlyapgio(F,Q(:,:,ST(1)));

% The sample log-likelihood function
%--------------------------------------------------------------
lik_mat = zeros(t,1);  store_beta=zeros(NY,t); store_P=zeros(NY,NY,t);

for j=1:t
  
  %Prediction -------------------------------------------------
  beta_tl = F*beta_ll;
  P_tl=F*P_ll*F'+Q(:,:,ST(j));
  vt=xdd(j,:)'- MU - H*beta_tl; %prediction error (r x 1) vector
  ft=H*P_tl*H'+R; %variance of forecast error (r x r) matrix
  
  %Updating ---------------------------------------------------
  beta_tt = beta_tl+P_tl*H'/ft*vt;
  P_tt=P_tl-P_tl*H'/ft*H*P_tl;

  %likelihood -------------------------------------------------
  if j>t0
    lik_mat(j,1)=-0.5*(nvarobs*log(2*pi) + log(det(ft)) + vt'/ft*vt);
  end
  
  beta_ll         = beta_tt;
  store_beta(:,j) = beta_tt;
  
  P_ll           = P_tt;
  store_P(:,:,j) = P_tt;
    
end

lnlik = sum(lik_mat(1:t,1));



