function [y, lik_mat, pr_st0, pr_stl0] = kn_filter(xdd, F, impact, SDX, P, H, CC, nstates)

% Initializing Markov-switching framework and Kalman filter
%--------------------------------------------------------------
% 
t       = size(xdd,1);
nvar    = size(F,1); 
nvarobs = size(xdd,2);

% variance-covariance matrix of the disturbance vector
Q = cell(nstates,1);

for st=1:nstates
	QQ     = SDX(:,:,st)^2;	
	Q{st}  = impact*QQ*impact';
end

% filtered probabilities
pr_st0  = zeros(t,nstates); % save all Pr[S_t=0|Y_{t}]
pr_stl0 = zeros(t,nstates); % save all Pr[S_t=0|Y_{t-1}]

P_ll    = zeros(nvar,nvar,nstates);

for g=1:nstates
     P_ll(:,:,g) = lyapunov_symm(F, Q{g}, 1.0, 1e-6);
end

beta_ll        = zeros(nvar, nstates);
beta_tl        = cell(nstates*nstates,t);
P_tl           = cell(nstates*nstates,t);
vt             = cell(nstates*nstates,t);
ft             = cell(nstates*nstates,t);

beta_tt        = zeros(nvar,nstates*nstates);
P_tt           = zeros(nvar,nvar,nstates*nstates);
conDens        = cell(nstates*nstates,1);
WeightcondDens = zeros(nstates*nstates,1);
JointProb      = zeros(nstates*nstates,1);
detft          = cell(nstates*nstates,1);

%Steady-state probabilities 
phi_ss     = [eye(nstates) - P; ones(1,nstates)]; 
y_ss       = [zeros(nstates,1);1];
Prob_ss = fn_ols(y_ss,phi_ss); % see Kim and Nelson (1999), page 70, section 4.3.2
Prob    = vecr(Prob_ss(:,ones(1,nstates)));

% Measurement equation y_t = MU + H_t.B_t 
%--------------------------------------------------------------
R  = 0;
t0 = 4;
MU = H*CC;

% The sample log-likelihood
%--------------------------------------------------------------
lik_mat = zeros(t,1);

for k=1:t
	
  %Probability terms Pr[S_t=j,S_{t-1}=i|Y_{t-1}] --------------
  Prob_conDens = P(:).*Prob;

    for i=1:nstates
        for j=1:nstates

m = (i-1)*nstates+j;
  
  %Prediction -------------------------------------------------
  beta_tl{m,k} = F*beta_ll(:,i);
  P_tl{m,k} = F*P_ll(:,:,i)*F'+Q{j};
  vt{m,k} = xdd(k,:)'-MU-H*beta_tl{m,k}; %prediction error (r x 1) vector
  ft{m,k}=H*P_tl{m,k}*H'+R; %variance of forecast error (r x r) matrix

  %Updating ---------------------------------------------------
  XXX = P_tl{m,k}*H'/ft{m,k};
  beta_tt(:,m) = beta_tl{m,k}+XXX*vt{m,k};
  P_tt(:,:,m)=P_tl{m,k}-XXX*H*P_tl{m,k};

  %Determinant of f_t -----------------------------------------
  detft{m} = log(det(ft{m,k}));

  %Conditional densities Pr[Y_t|Y_{t-1}, S_t] -----------------
  conDens{m}= -0.5*(nvarobs*log(2*pi) + detft{m}+ vt{m,k}'/ft{m,k}*vt{m,k});

  %Conditional densities times weight Pr[S_t,Y_t|Y_{t-1}] -----
  WeightcondDens(m,1) = conDens{m} + log(Prob_conDens(m));

        end
    end

  %log of Weighted average of conditional densities Pr[Y_t|Y_{t-1}] --
  dens = log(sum(exp(WeightcondDens - max(WeightcondDens)))) + max(WeightcondDens);

  %log likelihood for each date
  if k>t0
    lik_mat(k,1) = dens;
  end

for l=1:nstates*nstates
  %Update the probability terms Pr[S_t,S_{t-1}|Y_t] -----------
  JointProb(l,1) = exp(WeightcondDens(l,1) - dens);
end

  %Update the probability terms Pr[S_t|Y_t] -------------------
  Pr = sum(reshape(JointProb,nstates,nstates),2);

  %build the probability terms Prob for the next period
  Prob = vecr(Pr(:,ones(1,nstates)));

  %get the probability terms Pr[S_t=0|Y_{t-1}] for Kim filter
  tmp = sum(reshape(Prob_conDens,nstates,nstates),2);

  [beta_ll, P_ll] = collapsing_terms(P_tt,JointProb,beta_tt,Pr,nstates,nvar);

  %Hamilton filter --------------------------------------------
  pr_st0(k,:)  = Pr';  %Pr[S_t=0|Y_t]
  pr_stl0(k,:) = tmp'; %Pr[S_t=0|Y_{t-1}]

end	

y = sum(lik_mat(1:t,1));

