function [y, lik_mat, pr_st0, pr_stl0] = lik_ms_sbvar(xdd, F, Sigma, Q, nstates, nlags)
% Compute the sample log-likelihood function
% Markov-switching reduced-form VAR form 
%
% Stephane Lhuissier 2015

t    = size(xdd,1);
nvar = size(xdd,2);

%[xtx,xty,yty,fss,xx,yy,ncoef,xr,Bh,e] = fn_dataxy(nvar,nlags,xdd,0,0,1);

yy   = xdd(2:end,:)';
xx   = xdd(1:end-1,:)';

% Initializing Markov-switching framework
%--------------------------------------------------------------
%Steady-state probabilities PR[S_t=0] and PR[S_t=1]
Prob_ss = 1/nstates*ones(nstates,1);
Prob    = vecr(Prob_ss(:,ones(1,nstates))); % A 4x1 matrix

pr_st0  = zeros(t-nlags,nstates); % save all Pr[S_t=0|Y_{t}]
pr_stl0 = zeros(t-nlags,nstates); % save all Pr[S_t=0|Y_{t-1}]

% The sample log-likelihood function
%--------------------------------------------------------------
lik_mat = zeros(t-nlags,1);
lcond   = zeros(nstates,1);
fss     = -0.5*nvar*log(2*pi);

invS    = zeros(nvar,nvar,nstates);
detS    = zeros(nstates,1);

for j_iter=1:nstates
    %
    invS(:,:,j_iter) = inv(Sigma(:,:,j_iter)); 
    [~,Su]           = lu(invS(:,:,j_iter));
    detS(j_iter,1)   = sum(log(abs(diag(Su))));
end


for k=1:t-nlags

    Prob_j = Q(:).*Prob; % A 4x1 vector  
  
    for j=1:nstates
        %
	    lcond(j,:) = 0.5*detS(j,1) - 0.5*((yy(:,k)-F*xx(:,k))'*invS(:,:,j)*((yy(:,k)-F*xx(:,k))));
    end

	vec_lcond = lcond(:,ones(1,nstates));

	%conditional densities, weight being Pr[S_{t}=j,S_{t-1}=i|Y_{t-1}]
	densj = fss + vec_lcond(:) + log(Prob_j);


	%log of Weighted average of conditional densities Pr[Y_t|Y_{t-1}] --
	maxdensj = max(densj);
	dens = log(sum(exp(densj - maxdensj))) + maxdensj;

	%log likelihood for each date
	lik_mat(k,1) = dens;

	%Update the probability terms Pr[S_t,S_{t-1}|Y_t] -----------
	JointProb = exp(densj - dens);

	%Update the probability terms Pr[S_t|Y_t] -------------------
	Pr = sum(reshape(JointProb,nstates,nstates),2);

	%build the probability terms Prob for the next period
	Prob = vecr(Pr(:,ones(1,nstates)));

	%get the probability terms Pr[S_t=0|Y_{t-1}] for Kim filter
	tmp = sum(reshape(Prob_j,nstates,nstates),2);

	pr_st0(k,:)  = Pr';  % Pr[S_t=0|Y_t]
	pr_stl0(k,:) = tmp'; % Pr[S_t=0|Y_{t-1}]

end

y = sum(lik_mat(:,1));
