function y = tz_loginversegammapdf(x, a_invgam, b_invgam)
%-----------------------------------------------------------------------------------
%//------------------------ Inverse-Gamma distribution ------------------------------//
%//--- p(x) = ( b^a/Gamma(a) ) x^(-a-1) exp(-b/x) for a>0 and b>0.
%//---    where a is shape and b is scale parameter.
%//--- E(x) = b/(a-1) for a>1;  var(x) = b^2/( (a-1)^2*(a-2) ) for a>2;
%//--- Noninformative distribution: a,b -> 0.
%//--- How to draw: (1) draw z from Gamma(a,b); (2) let x=1/z.
%//-----------------------------------------------------------------------------------
%
% by Stephane Lhuissier, 2015

%double tz_loginversegammapdf(double _x, double _a, double _b)
%{
   %//This density is always finite.
   %//If a < 1.0, 1st moment does not exist,
   %//   a < 2.0, 2nd moment does not exist,
   %//   a < 3.0, 3rd moment does not exist,
   %//   a < 4.0, 4th moment does not exist.

%   if (_x < 0.0 || _a <= 0.0 || _b <= 0.0)  return (-NEARINFINITY);
%      //fn_DisplayError("cstz.c/tz_loginversegammapdf(): x, a, and b must be positive");
%
%   return ( _a*log(_b) - fn_gammalog(_a) - (_a+1.0)*log(_x) - _b /_x );
%}
%
% _a*log(_b) - fn_gammalog(_a) - (_a+1.0)*log(_x) - _b /_x ;

y = a_invgam*log(b_invgam) - gammaln(a_invgam) - (a_invgam+1)*log(x) - b_invgam./x;
 