clear all; clc;

addpath 'W:\2_Recherche\LHUISSIER\T828085\csminwel-dynare-4.2.2'
addpath 'library'

% Input and Data
%--------------------------------------------------------------
name_file           = 'ms';
options_.name_lik   = strcat(name_file,'_lik');
options_.name_prior = strcat(name_file,'_prior');

% load data
load 'AWMdatabase_70Q2_12Q1.mat'
% choose the time series
xdd=lgdp;    
% quarters (4) or months (12)
q_m = 4;
% first year to use in estimation
yrBin=1970;
% first quarter or month to use in estimation
qmBin=2;
% last year to use in estimation
yrFin=2012;
% last quarter or month to use in estimation
qmFin=1;
% first year to use in estimation
yrStart=1970;
% first quarter or month to use in estimation
qmStart=2;
% last year to use in estimation
yrEnd=2012;
% last quarter or month to use in estimation
qmEnd=1;
% number of data points
nData=(yrFin-yrBin)*q_m + (qmFin-qmBin+1);
% number of data points in estimation sample
nSample=(yrEnd-yrStart)*q_m + (qmEnd-qmStart+1);
% number of periods not used at beginning of data (non-negative number)
nStart=(yrStart-yrBin)*q_m + (qmStart-qmBin);
% number of periods not used at end of data (non-positive number)
nEnd=(yrEnd-yrFin)*q_m + (qmEnd-qmFin);
%
xdatae = xdd(nStart+1:nData+nEnd,:);
options_.data = [xdatae(2:end),ones(nSample-1,1),xdatae(1:end-1)];
%
[date, n] = fn_calyrqm(q_m, [yrStart qmStart], [yrEnd qmEnd]);
qdates=zeros(n,1);
for ki=1:n
   qdates(ki) = date(1,1) + (date(1,2)+ki-2)/q_m;
end

% Priors
%--------------------------------------------------------------
ms_prior_lag1;
options_.pshape   = prior(:,1);
options_.sp       = prior(:,2);
options_.pmean    = prior(:,3);
options_.pstdd    = prior(:,4);
options_.trspec   = prior(:,5:8);
options_.mask     = prior(:,9);
options_.fix      = prior(:,10);
options_.maskinv  = 1-options_.mask;
options_.poscal   = find(options_.mask); 
options_.posest   = (1:length(options_.sp)); 
options_.posest   = find( ismember(options_.posest(:),options_.poscal(:)) == 0 ); 
x0                = options_.sp(options_.posest);

% Input for C.S optimizer
%--------------------------------------------------------------
options_.nparam  = size(options_.sp,1);
options_.post    = 1;
H0               = 1e-4*eye(options_.nparam);
H0               = H0(options_.posest,options_.posest);
crit             = 1.0e-6;
nit              = 10000;
gradient_method  = 2;
gradient_epsilon = 1e-6;
tic
% Numerical optimization by Christopher Sims : BFGS optimizer
[fh,xh,gh,H,itct,fcount,retcodeh] = csminwel1('min_post',invtrans(x0,options_.trspec(options_.posest,:)),H0,[],crit,nit, gradient_method, gradient_epsilon, options_)
toc

xh=trans(xh,options_.trspec(options_.posest,:));

Postmode = zeros(options_.nparam,1);
Postmode(options_.posest)=xh;
Postmode=Postmode.*(1-options_.mask)+options_.fix.*options_.mask;

% Recover real-time probabilities and condxhitional likelihood
[lnlik, pr_st0, pr_stl0, Q] =  feval(options_.name_lik, Postmode, options_.data(:,1), options_.data(:,2:end));

%
lnprior = priodens(xh, options_.pmean(options_.posest), ...
                       options_.pstdd(options_.posest), ...
                       options_.pshape(options_.posest));
% Kim filter 
[pr_sT, pr_joint_sT] = SmoothedProbabilities(pr_st0, pr_stl0, Q);

% Plot smoothed probabilities
figure
plot(qdates(2:end),pr_sT(:,1),'b','LineWidth',1.5)
xlim([qdates(2) qdates(end)])
ylim([-0.05 1.05])
xlabel('years')
ylabel('prob.')
box off

save('Results/posterior_mode_lag1.mat','Postmode','pr_sT','pr_st0','options_','qdates')

disp(' ')
disp('----------------------------------------------------------------')
disp('Empirical moments')
disp('----------------------------------------------------------------')
cnames={'Mode'};
pnames={'constant','scale(1)','scale(2)','p11','p22','phi_1'};
r=printresults3([xh],cnames,pnames); disp(r)

