

function [lik, pr_st0, pr_stl0, Q, lik_mat] = ms_lik(param,yt,xt)

t  = size(yt,1);

scale    = [param(2);param(3)]; % std

Q = [param(4), 1-param(5)
    1-param(4), param(5) ]; % Transition matrix

phi = [param(1);param(6)];

ns = size(Q,1);

% Initializing Markov-switching framework
%--------------------------------------------------------------
%Steady-state probabilities PR[S_t=0] and PR[S_t=1]
Prob = 1/ns*ones(ns,1);

pr_st0  = zeros(t,ns); % save all Pr[S_t=0|Y_{t}]
pr_stl0 = zeros(t,ns); % save all Pr[S_t=0|Y_{t-1}]


% The sample log-likelihood function
%--------------------------------------------------------------
lik_mat = zeros(t,1);
lcond   = zeros(ns,1);

for k=1:t
    
    Prob_j = Q*Prob;

    for j=1:ns   
        lcond(j,:) = - 0.5*log(2*pi) - log(scale(j)) - 0.5*((yt(k,1)-xt(k,:)*phi).^2)./(scale(j).^2);
    end

	%conditional densities, weight being Pr[S_{t}=j,S_{t-1}=i|Y_{t-1}]
	densj = lcond + log(Prob_j);

	%log of Weighted average of conditional densities Pr[Y_t|Y_{t-1}] --
	maxdensj = max(densj);
	dens = log(sum(exp(densj - maxdensj))) + maxdensj;

	%log likelihood for each date
	lik_mat(k,1) = dens;

	%Update the probability terms Pr[S_t,S_{t-1}|Y_t] -----------
	Prob = exp(densj - dens);

	pr_st0(k,:)  = Prob';   % Pr[S_t=0|Y_t]
	pr_stl0(k,:) = Prob_j'; % Pr[S_t=0|Y_{t-1}]
                                                                                         
end

lik = sum(lik_mat(:,1));

