clear all; clc;

addpath 'library/'
        
nlags = 1;
n     = 11000;
nuse  = 10;
nburn = 0.10*n;

load('Results/posterior_mode_lag1.mat','Postmode','pr_sT','pr_st0','options_','qdates');

YY=options_.data;
yt=options_.data(:,1);
xt=options_.data(:,2:end);

param = Postmode;

sig = [param(2);param(3)];
p11 = param(4);
p22 = param(5);
phi = [param(1);param(6)];

% Initial values 
sigd = sig;
phid = phi;
p11d = p11;
p22d = p22;

% priors
t0 = zeros(2,1);
R0 = (5.00*eye(2)).^2;

p_pmean = 0.80;
p_pstdd = 0.10;

s_invgam  = 0.1679;
nu_invgam = 2.0395;

% Gibbs-sampling procedure
phi_store = zeros(n,2);
sig_store = zeros(n,2);
c_store   = zeros(n,1);
p11_store = zeros(n,1);
p22_store = zeros(n,1);
s_store   = zeros(size(yt,1),n);

tic
for i=1:n

% STEP 0.
sd = gbs_s(param,yt,xt);

% STEP 1.
qd = gbs_q(p_pmean,p_pstdd,sd);

% STEP 2.
phid = gbs_mu(yt,xt,sd,sigd,t0,R0);

% STEP 3.
sigd = gbs_sig(yt,xt,sd,phid,s_invgam,nu_invgam);

% allocate to param
param([1,6])=phid;
param(2:3)=sigd;
param(4)=qd(1,1);
param(5)=qd(2,2);

% STORE
s_store(:,i)=sd;
phi_store(i,:)=phid';
sig_store(i,:)=sigd';
p11_store(i)=qd(1,1);
p22_store(i)=qd(2,2);
end

fprintf('Time estimation is:  %s.\n',secs2hms(toc));

figure;
subplot(2,3,1)
plot(sig_store(:,1)')
hold on
plot(sig(1)*ones(n,1),'r')
title('scale(1)')
xlim([1 n])
%
subplot(2,3,2)
plot(sig_store(:,2)')
hold on
plot(sig(2)*ones(n,1),'r')
title('scale(1)')
xlim([1 n])
%
subplot(2,3,3)
plot(p11_store)
hold on
plot(p11*ones(n,1),'r')
title('p11')
xlim([1 n])
%
subplot(2,3,4)
plot(p22_store)
hold on
plot(p22*ones(n,1),'r')
title('p22')
xlim([1 n])
%
subplot(2,3,5)
plot(phi_store(:,1)')
hold on
plot(phi(1)*ones(n,1),'r')
title('constant')
xlim([1 n])
%
subplot(2,3,6)
plot(phi_store(:,2)')
hold on
plot(phi(2)*ones(n,1),'r')
title('phi_1')
xlim([1 n])

n = (n-nburn)/nuse;

phidraw = forevery(phi_store(nburn+1:end,:),nuse);
sigdraw = forevery(sig_store(nburn+1:end,:),nuse);
p11draw = forevery(p11_store(nburn+1:end),nuse);
p22draw = forevery(p22_store(nburn+1:end),nuse);
sdraw   = forevery(s_store(:,nburn+1:end)',nuse);

Postdraw = [phidraw(:,1) sigdraw p11draw p22draw phidraw(:,2)];

Postdraw_med = median(Postdraw,1);
Postdraw_mea = mean(Postdraw,1);
Postdraw_lim = hpdint(Postdraw,0.90,0);

disp(' ')
disp('----------------------------------------------------------------')
disp('Empirical moments')
disp('----------------------------------------------------------------')
cnames={'Mode','Mean','Median','Lower', 'Upper'};
pnames={'constant','std(1)','std(2)','p11','p22','phi_1'};
out=[Postmode,Postdraw_mea',Postdraw_med',Postdraw_lim'];
r=printresults3(out,cnames,pnames); disp(r)


